﻿Param(
    [Parameter(Mandatory=$false)]
    [String]
    $CertFriendlyName = "BA Insight Self Signed Certificate",

    [Parameter(Mandatory=$false)]
    [String]
    $IssuedTo
)

# Set optional parameters (only available in recent PowerShell versions).
$createCertCmd = Get-Command New-SelfSignedCertificate

# Check for required PowerShell version.
if (-not $createCertCmd.Parameters.ContainsKey("FriendlyName")) {
    Write-Host ("ERROR: Current version of PowerShell API for certificates is not supported. " +
               "Please run the script on a computer running Windows 10, Windows 2016 or newer. " +
               "The generated certificates can then be used on other (older) operating systems.") -ForegroundColor "Red"
    exit 1
}

# Create new certificate. New self-signed certificates can only be created in the user's store.
Write-Debug "Creating a new self-signed certificate in the current user's local store..."
if ($IssuedTo -eq $null) { 
    $hostName = [System.Net.Dns]::GetHostByName($env:computerName).HostName
}
else {
    $hostName = $IssuedTo
}
$currentUserCert = New-SelfSignedCertificate -CertStoreLocation "Cert:\LocalMachine\My" `
                                             -DnsName $hostName `
                                             -NotAfter ((Get-Date).AddYears(100)) `
                                             -FriendlyName $CertFriendlyName `
                                             -Subject "CN=$hostName, CN=$CertFriendlyName" `
                                             -KeySpec KeyExchange

# Export the certificate to a CER file (for subsequent import into other stores, e.g. Azure).
Write-Debug "Exporting the certificate to a file..." 
$certExportResult = Export-Certificate -Cert $currentUserCert -FilePath ($CertFriendlyName + ".cer")

# Export the certificate to a PFX file (for subsequent import into local certificate store).
$password = Read-Host "Type a password which will be used in the generated PFX certificate: " -AsSecureString
$pfxExportResult = Export-PfxCertificate -Cert $currentUserCert `
                                         -FilePath ($CertFriendlyName + ".pfx") `
                                         -Password $password

# Delete the certificate form the user store.
Write-Debug "Removing the certificate from the current user's local store..." 
Remove-Item -LiteralPath $currentUserCert.PSPath

Write-Host "SUCCESS: Self-signed certificate created." -ForegroundColor "Green"
Write-Host "Certificate data: `r`n$rootCert" -ForegroundColor "Green"
Write-Host "The certificate is exported to $currentUserCert" -ForegroundColor "Green"